package javax.swing.plaf.metal;

import java.beans.PropertyChangeEvent;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.basic.BasicScrollPaneUI;

public class MetalScrollPaneUI extends BasicScrollPaneUI
{
    private PropertyChangeListener scrollBarSwapListener;
    
    public MetalScrollPaneUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new MetalScrollPaneUI();
    }
    
    @Override
    public void installUI(JComponent x) {
        super.installUI(x);
        JScrollPane scrollPane = (JScrollPane)x;
        this.updateScrollbarsFreeStanding();
    }
    
    @Override
    public void uninstallUI(JComponent c) {
        super.uninstallUI(c);
        JScrollPane scrollPane = (JScrollPane)c;
        JScrollBar horizontalScrollBar = scrollPane.getHorizontalScrollBar();
        JScrollBar verticalScrollBar = scrollPane.getVerticalScrollBar();
        if (horizontalScrollBar != null) {
            horizontalScrollBar.putClientProperty("JScrollBar.isFreeStanding", null);
        }
        if (verticalScrollBar != null) {
            verticalScrollBar.putClientProperty("JScrollBar.isFreeStanding", null);
        }
    }
    
    public void installListeners(JScrollPane c) {
        super.installListeners(c);
        c.addPropertyChangeListener(this.scrollBarSwapListener = this.createScrollBarSwapListener());
    }
    
    @Override
    protected void uninstallListeners(JComponent c) {
        super.uninstallListeners(c);
        c.removePropertyChangeListener(this.scrollBarSwapListener);
    }
    
    @Deprecated
    public void uninstallListeners(JScrollPane c) {
        super.uninstallListeners(c);
        c.removePropertyChangeListener(this.scrollBarSwapListener);
    }
    
    private void updateScrollbarsFreeStanding() {
        if (this.scrollpane == null) {
            return;
        }
        Boolean b;
        if ((this.scrollpane.getBorder() instanceof MetalBorders.ScrollPaneBorder)) {
            b = Boolean.FALSE;
        }
        else {
            b = Boolean.TRUE;
        }
        JScrollBar horizontalScrollBar = this.scrollpane.getHorizontalScrollBar();
        if (horizontalScrollBar != null) {
            horizontalScrollBar.putClientProperty("JScrollBar.isFreeStanding", b);
        }
        JScrollBar verticalScrollBar = this.scrollpane.getVerticalScrollBar();
        if (verticalScrollBar != null) {
            verticalScrollBar.putClientProperty("JScrollBar.isFreeStanding", b);
        }
        return;
    }
    
    protected PropertyChangeListener createScrollBarSwapListener() {
        return new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
                String propertyName = propertyChangeEvent.getPropertyName();
                if (propertyName.equals("verticalScrollBar") || propertyName.equals("horizontalScrollBar")) {
                    JScrollBar scrollBar = (JScrollBar)propertyChangeEvent.getOldValue();
                    if (scrollBar != null) {
                        scrollBar.putClientProperty("JScrollBar.isFreeStanding", null);
                    }
                    JScrollBar scrollBar2 = (JScrollBar)propertyChangeEvent.getNewValue();
                    if (scrollBar2 != null) {
                        scrollBar2.putClientProperty("JScrollBar.isFreeStanding", Boolean.FALSE);
                    }
                }
                else if ("border".equals(propertyName)) {
                    MetalScrollPaneUI.access$000(MetalScrollPaneUI.this);
                }
            }
        };
    }
    
    static void access$000(MetalScrollPaneUI metalScrollPaneUI) {
        metalScrollPaneUI.updateScrollbarsFreeStanding();
    }
}
